# -*- coding: utf-8 -*-
"""
Created on Thu Sep 21 11:57:41 2017

@author: ppradeep

----------------------------------------------------------------------------------------
This script runs a descriptive analysis on the input data file
Input:
1. Data file with effect level data per chemical
2. PubChem fingerprints
3. CDK descriptors
4. PADel descriptors
Outputs:
1. Effect level counts per chemical per study type and species combination
2. Effect level range per chemical per study type and species combination
3. Effect level variance per chemical per study type and species combination
4. Effect level standard deviation per chemical per study type and species combination
5. Absolute counts per chemical per study type and species combination
----------------------------------------------------------------------------------------
"""

#%%

## Import packages
import warnings
warnings.filterwarnings('ignore')

import pandas as pd
import matplotlib.pyplot as plt
import numpy as np
#import math
  
path = 'Y:/Projects/ToxvalModels/'
path = 'C:/Users/ppradeep/Desktop/ToxvalModels/'
#%%
# Read the input data file
dataAll = pd.read_excel(path+'data/toxval_pod_summary_human health_2019-08-20.xlsx', index_col = 'casrn')

#%%
study_comb = [
              [['chronic', 'cancer'], ['rat']],
              [['chronic', 'cancer'], ['mouse']],
              #[['chronic', 'cancer'], ['rabbit']],
              [['chronic', 'cancer'], ['rat','mouse']],
              [['subchronic', 'repeat dose', 'short-term'], ['rat']],
              [['subchronic', 'repeat dose', 'short-term'], ['mouse']],
              [['subchronic', 'repeat dose', 'short-term'], ['rabbit']],
              [['subchronic', 'repeat dose', 'short-term'], ['rat','mouse', 'rabbit']],
              [['reproductive', 'multigenerational reproductive'], ['rat']],
              [['reproductive', 'multigenerational reproductive'], ['mouse']],
              #[['reproductive', 'multigenerational reproductive'], ['rabbit']],
              [['reproductive', 'multigenerational reproductive'], ['rat','mouse']],
              [['developmental','reproductive developmental'], ['rat']],
              [['developmental','reproductive developmental'], ['mouse']],
              [['developmental','reproductive developmental'], ['rabbit']],
              [['developmental','reproductive developmental'], ['rat', 'mouse', 'rabbit']],    
              [['subacute'], ['rat']],
              [['chronic', 'cancer', 'subchronic', 'repeat dose', 'short-term', 'reproductive', 'multigenerational reproductive', 'developmental', 'reproductive developmental', 'subacute'], ['rat', 'mouse', 'rabbit']]
              ]

labels = [    [['CHR'], ['rat']],
              [['CHR'], ['mouse']],
              #[['CHR'], ['rabbit']],
              [['CHR'], ['rat','mouse']],
              [['SUB'], ['rat']],
              [['SUB'], ['mouse']],
              [['SUB'], ['rabbit']],
              [['SUB'], ['rat','mouse', 'rabbit']],
              [['REP'], ['rat']],
              [['REP'], [' mouse']],
              #[['REP'], [' rabbit']],
              [['REP'], ['rat','mouse']],
              [['DEV'], ['rat']],
              [['DEV'], ['mouse']],
              [['DEV'], ['rabbit']],
              [['DEV'], ['rat','mouse', 'rabbit']],    
              [['SAC'], ['rat']],
              [['ALL'], ['rat', 'mouse', 'rabbit']]
              ]

#%%
## Read PubChem fingerprints and CDK descriptors generated in KNIME
#toxprints = pd.read_csv(path+'data/toxval_toxprints.csv', index_col = 'M_NAME')
pubchem = pd.read_csv(path+'data/toxval_pubchem.csv', index_col = 'row ID')
padel = pd.read_csv(path+'data/toxval_padel.csv', index_col = 'Name').dropna()
cdk = pd.read_csv(path+'data/toxval_cdk.csv', index_col = 'row ID')

#%%
data_file = dataAll.loc[padel.index]

#%%
## Number of data points and unique chemicals per category type.
for comb in study_comb:
    data = data_file[(data_file['risk_assessment_class'].isin(comb[0])) \
               & (data_file['toxval_units'] == 'mg/kg-day') \
               & (data_file['toxval_type_supercategory'] == 'Point of Departure') \
               & (~data_file['toxval_type'].isin(['ERCL10','NOTEL', \
                  'EC0', 'EC5', 'EC10', 'EC20', 'EC50', 'EC90', 'EC100',\
                  'IC50', 'IC100'\
                  ])) \
               & (data_file['species_common'].isin(comb[1]))]\
                  [['toxval_numeric', 'risk_assessment_class', 'species_common']] 
    print(comb, len(data), len(data.index.unique()))

               
#%%
data.groupby('risk_assessment_class').count()*100/len(data)

#%%
## Distribution of DATA
fig, ax = plt.subplots(4,4,figsize=(16,12), dpi = 300)
plt.subplots_adjust(hspace=0.2, wspace=0.2)

for i, comb in enumerate(study_comb[0:16]):
    data = data_file[(data_file['risk_assessment_class'].isin(comb[0])) \
               & (data_file['toxval_units'] == 'mg/kg-day') \
               & (data_file['toxval_type_supercategory'] == 'Point of Departure') \
               & (~data_file['toxval_type'].isin(['ERCL10','NOTEL', \
                  'EC0', 'EC5', 'EC10', 'EC20', 'EC50', 'EC90', 'EC100',\
                  'IC50', 'IC100',\
                  ])) \
               & (data_file['species_common'].isin(comb[1]))][['toxval_numeric']]  
    Y = np.log10(data) 
    label = '%s | %s' %(str(labels[i][0]).strip("[,]").replace("'",""), str(labels[i][1]).strip("[,]").replace("'",""))

    plt.subplot(4,4,i+1)
    plt.hist(Y['toxval_numeric'].values, bins = 50, alpha = 0.75)
    plt.annotate(label, [0.90*plt.xlim()[0], 0.90*plt.ylim()[1]], fontsize=14)
    plt.annotate('N=%d'%len(Y), [0.90*plt.xlim()[0], 0.75*plt.ylim()[1]], fontsize=14)
    plt.xticks(fontsize = 12)#, rotation = 90)
    plt.yticks(fontsize = 12)

fig.text(0.5, 0.05, 'Effect level values$_{tr}$', ha='center', size = 32)
fig.text(0.05, 0.5, 'Frequency', va='center', size = 32, rotation='vertical')

plt.savefig(path+'/output/Data.png', bbox_inches='tight')


#%%
## Distribution of Number of data points per chemical
fig, ax = plt.subplots(4,4,figsize=(16,12), dpi = 300)
plt.subplots_adjust(hspace=0.2, wspace=0.2)

for i, comb in enumerate(study_comb):#[0:16]):
    data = data_file[(data_file['risk_assessment_class'].isin(comb[0])) \
               & (data_file['toxval_units'] == 'mg/kg-day') \
               & (data_file['toxval_type_supercategory'] == 'Point of Departure') \
               & (~data_file['toxval_type'].isin(['ERCL10','NOTEL', \
                  'EC0', 'EC5', 'EC10', 'EC20', 'EC50', 'EC90', 'EC100',\
                  'IC50', 'IC100',\
                  ])) \
               & (data_file['species_common'].isin(comb[1]))][['toxval_numeric']]  
    Y = np.log10(data) 
    label = '%s | %s' %(str(labels[i][0]).strip("[,]").replace("'",""), str(labels[i][1]).strip("[,]").replace("'",""))

    plt.subplot(4,4,i+1)
    plt.hist(Y.groupby(Y.index).count()['toxval_numeric'].values, bins = 20, alpha = 0.5)
    plt.annotate(label, [0.05*plt.xlim()[1], 0.90*plt.ylim()[1]], fontsize=14)
    plt.xticks(fontsize = 12)#, rotation = 90)
    plt.yticks(fontsize = 12)
    plt.xlim([0, plt.xlim()[1]])

fig.text(0.5, 0.05, 'Number of effect level values per chemical', ha='center', size = 30)
fig.text(0.05, 0.5, 'Frequency', va='center', size = 28, rotation='vertical')

plt.savefig(path+'/output/DataStudySources.png', bbox_inches='tight') 

#%%
## Distribution of Range of POD values per chemical
fig, ax = plt.subplots(4,4,figsize=(16,12), dpi = 300)
plt.subplots_adjust(hspace=0.2, wspace=0.2)

for i, comb in enumerate(study_comb):#[0:16]):
    data = data_file[(data_file['risk_assessment_class'].isin(comb[0])) \
               & (data_file['toxval_units'] == 'mg/kg-day') \
               & (data_file['toxval_type_supercategory'] == 'Point of Departure') \
               & (~data_file['toxval_type'].isin(['ERCL10','NOTEL', \
                  'EC0', 'EC5', 'EC10', 'EC20', 'EC50', 'EC90', 'EC100',\
                  'IC50', 'IC100',\
                  ])) \
               & (data_file['species_common'].isin(comb[1]))][['toxval_numeric']] 
               
    label = '%s | %s' %(str(labels[i][0]).strip("[,]").replace("'",""), str(labels[i][1]).strip("[,]").replace("'",""))
    Y = np.log10(data) 
    
    Y_range = Y.groupby(Y.index).max()-Y.groupby(Y.index).min()
    Y_range = Y_range[Y_range != 0]
    mean_range = Y_range.mean()
    std_range = Y_range.std()
    
    plt.subplot(4,4,i+1)
    plt.hist(Y_range.values, bins = 20, alpha = 0.5)

    plt.annotate(label, [0.02*plt.xlim()[1], 0.90*plt.ylim()[1]], fontsize=16) #
    plt.annotate('$\mu=%0.2f, \sigma=%0.2f$' %(mean_range, std_range), [0.02*plt.xlim()[1], 0.80*plt.ylim()[1]], fontsize=16)
    
    plt.xticks(fontsize = 12)#, rotation = 90)
    plt.yticks(fontsize = 12)

    plt.xlim([0, plt.xlim()[1]])
fig.text(0.5, 0.05, 'Effect level range per chemical ($log_{10} mg/kg/day$)', ha='center', size = 28)
fig.text(0.05, 0.5, 'Frequency', va='center', size = 28, rotation='vertical')

plt.savefig(path+'/output/DataRange.png', bbox_inches='tight')

#%%
# Distribution of variance of POD values for each chemical
fig, ax = plt.subplots(4,4,figsize=(16,12), dpi = 300)
plt.subplots_adjust(hspace=0.2, wspace=0.2)

for i, comb in enumerate(study_comb):#[0:16]):
    data = data_file[(data_file['risk_assessment_class'].isin(comb[0])) \
               & (data_file['toxval_units'] == 'mg/kg-day') \
               & (data_file['toxval_type_supercategory'] == 'Point of Departure') \
               & (~data_file['toxval_type'].isin(['ERCL10','NOTEL', \
                  'EC0', 'EC5', 'EC10', 'EC20', 'EC50', 'EC90', 'EC100',\
                  'IC50', 'IC100',\
                  ])) \
               & (data_file['species_common'].isin(comb[1]))][['toxval_numeric']]            
    
    label = '%s | %s' %(str(labels[i][0]).strip("[,]").replace("'",""), str(labels[i][1]).strip("[,]").replace("'",""))
    Y = np.log10(data) 
    Y_var = Y.groupby(Y.index).var().dropna()
    Y_var = Y_var[Y_var != 0]
    mean_var = Y_var.mean()

    plt.subplot(4,4,i+1)
    plt.hist(Y_var.values, bins = 20, alpha = 0.5)

    plt.annotate(label, [0.02*plt.xlim()[1], 0.90*plt.ylim()[1]], fontsize=16) #
    plt.annotate('$\mu=%0.2f$' %(mean_var), [0.02*plt.xlim()[1], 0.80*plt.ylim()[1]], fontsize=16)
    
    plt.xticks(fontsize = 12)#, rotation = 90)
    plt.yticks(fontsize = 12)

    plt.xlim([0, plt.xlim()[1]])
    
fig.text(0.5, 0.05, 'Effect level variance per chemical ($log_{10} mg/kg/day$)', ha='center', size = 28)
fig.text(0.05, 0.5, 'Frequency', va='center', size = 28, rotation='vertical')

plt.savefig(path+'/output/DataVariance.png', bbox_inches='tight')


#%%
# Distribution of std of POD values for each chemical
fig, ax = plt.subplots(4,4,figsize=(16,12), dpi = 300)
plt.subplots_adjust(hspace=0.2, wspace=0.2)

for i, comb in enumerate(study_comb):
    data = data_file[(data_file['risk_assessment_class'].isin(comb[0])) \
               & (data_file['toxval_units'] == 'mg/kg-day') \
               & (data_file['toxval_type_supercategory'] == 'Point of Departure') \
               & (~data_file['toxval_type'].isin(['ERCL10','NOTEL', \
                  'EC0', 'EC5', 'EC10', 'EC20', 'EC50', 'EC90', 'EC100',\
                  'IC50', 'IC100',\
                  ])) \
               & (data_file['species_common'].isin(comb[1]))][['toxval_numeric']] 
    label = '%s | %s' %(str(labels[i][0]).strip("[,]").replace("'",""), str(labels[i][1]).strip("[,]").replace("'",""))
    Y = np.log10(data) 
    Y_std = Y.groupby(Y.index).std().dropna()
    Y_std = Y_std[Y_std != 0]
    mean_std = Y_std.mean()
    print(comb,mean_std)
    plt.subplot(4,4,i+1)
    plt.hist(Y_std.values, bins = 20, alpha = 0.5)

    plt.annotate(label, [0.02*plt.xlim()[1], 0.90*plt.ylim()[1]], fontsize=16) #
    plt.annotate('$\mu_{\sigma}=%0.2f$' %(mean_std), [0.02*plt.xlim()[1], 0.80*plt.ylim()[1]], fontsize=16)
    
    plt.xticks(fontsize = 12)#, rotation = 90)
    plt.yticks(fontsize = 12)

    plt.xlim([0, plt.xlim()[1]])
    #plt.ylim([0, 3])
    
fig.text(0.5, 0.05, 'Effect level standard deviation per chemical ($log_{10} mg/kg/day$)', ha='center', size = 28)
fig.text(0.05, 0.5, 'Frequency', va='center', size = 28, rotation='vertical')
    

plt.savefig(path+'/output/DataStd.png', bbox_inches='tight')

#%%
## Number of data points and unique chemicals per category type.
for comb in study_comb:
    data = data_file[(data_file['risk_assessment_class'].isin(comb[0])) \
               & (data_file['toxval_units'] == 'mg/kg-day') \
               & (data_file['toxval_type_supercategory'] == 'Point of Departure') \
               & (~data_file['toxval_type'].isin(['ERCL10','NOTEL', \
                  'EC0', 'EC5', 'EC10', 'EC20', 'EC50', 'EC90', 'EC100',\
                  'IC50', 'IC100',\
                  ])) \
               & (data_file['species_common'].isin(comb[1]))][['toxval_numeric']].dropna()
    print(comb, len(data), len(data.index.unique()))
    
#%%
## Data analysis summary
summary_df = pd.DataFrame()
for i, comb in enumerate(study_comb):
    data = data_file[(data_file['risk_assessment_class'].isin(comb[0])) \
               & (data_file['toxval_units'] == 'mg/kg-day') \
               & (data_file['toxval_type_supercategory'] == 'Point of Departure') \
               & (~data_file['toxval_type'].isin(['ERCL10','NOTEL', \
                  'EC0', 'EC5', 'EC10', 'EC20', 'EC50', 'EC90', 'EC100',\
                  'IC50', 'IC100',\
                  ])) \
               & (data_file['species_common'].isin(comb[1]))][['toxval_numeric']] 
    Y = np.log10(data) 
    Y_count = Y.groupby(Y.index).count()
    Y_range = Y.groupby(Y.index).max() - Y.groupby(Y.index).min()
    Y_var = Y.groupby(Y.index).var()
    Y_std = Y.groupby(Y.index).std()
    df = pd.DataFrame(index = Y_count.index)
    df['CASRN'] = df.index
    df['Count'] = Y_count
    df['Range'] = Y_range
    df['Variance'] = Y_var
    df['Std'] = Y_std
    df['Combination'] = str(comb)
    summary_df = summary_df.append(df)
    summary_df.index = summary_df['Combination'] 

summary_df.to_csv(path+'output/DataAnalysisSummary.csv')
#%%

for comb in summary_df.index.unique():
    std=summary_df.loc[comb,'Std'].dropna().mean()
    print(comb,std)
