# -*- coding: utf-8 -*-
"""
Created on Wed Nov 22 13:51:55 2017

@author: ppradeep

----------------------------------------------------------------------------------------
This script loads the user provided data file and uses the CASRNs to quesryt the DSSTox
database for QSAR-ready structure. The output is a SDF file with valid structures.
----------------------------------------------------------------------------------------

"""
#%%
## Import libraries
import mysql.connector
import pandas as pd
from rdkit import Chem
    
#%%
usr = '_dataminer'
pwd = 'pass'

# Set the project working directory
# Main directory structure -> data/code/output
path = 'Y:/Projects/ToxvalModels/'

#%%
## Read the input data file and extract unique CASRNs
data = pd.read_excel(path+'data/toxval_pod_summary_human health_2019-08-20.xlsx', index_col = 'casrn')
unique_cas = data.index.unique().tolist()

#%%
## Query the databased for QSAR-ready SMILES and SDF structures. Save the file.
writeSDF = Chem.SDWriter(path+'data/toxval_all_QSARready.sdf')
cnx = mysql.connector.connect(user=usr, password=pwd, host='mysql-res1.epa.gov', database='ro_stg_dsstox')
cursor = cnx.cursor()

for cas in unique_cas:
    try:
        cid = ''
        cas = str(cas)
        query1 = ("SELECT synonym_mv.identifier, compounds.id \
                FROM synonym_mv \
                INNER JOIN generic_substances ON generic_substances.id = synonym_mv.fk_generic_substance_id \
                LEFT OUTER JOIN generic_substance_compounds ON generic_substance_compounds.fk_generic_substance_id = synonym_mv.fk_generic_substance_id \
                LEFT OUTER JOIN compounds ON generic_substance_compounds.fk_compound_id = compounds.id \
                JOIN qc_levels ON generic_substances.fk_qc_level_id = qc_levels.id \
                WHERE synonym_mv.identifier = '%s' AND (NOT qc_levels.name = 'Incomplete') AND (synonym_type IN ('CAS-RN', 'Alternate CAS-RN', 'Deleted CAS-RN', 'Integated Source CAS-RN')) \
                GROUP BY synonym_mv.identifier, generic_substances.id, compounds.id \
                ORDER BY min(synonym_mv.rank)" %(cas))
        cursor.execute(query1)
        rows = cursor.fetchall()
        
        for row in rows:
            cid = row[1]
            
        smile = ''
        
        query2 = ("SELECT DISTINCT compounds.smiles, compound_relationships.fk_compound_id_predecessor\
                FROM compounds \
                JOIN compound_relationships \
                ON compounds.id = compound_relationships.fk_compound_id_successor\
                WHERE compound_relationships.fk_compound_id_predecessor = '%s' AND \
                compound_relationships.relationship = '2D-QSAR-Ready'" %(cid))
        
        cursor.execute(query2)
        rows = cursor.fetchall()
        
        for row in rows:
            smile = row[0]
            mol = Chem.MolFromSmiles(smile)
            mol.SetProp("_Name", cas)
            writeSDF.write(mol)
            break
    except:
        print(cas)

writeSDF.flush()
writeSDF.close()