# -*- coding: utf-8 -*-
"""
Created on Thu Feb 20 14:26:23 2020

@author: ppradeep
"""


## Import packages
import warnings
warnings.filterwarnings('ignore')

import pandas as pd
import numpy as np
from scipy import stats
import pickle
from numpy import mean, absolute 
import itertools

def mean_confidence_interval(data, confidence):
    mean=[]; lci = []; uci = []
    for idx in data.index:
        a = 1.0*np.array(data.loc[idx])
        n = len(a)
        m, se = np.mean(a), stats.sem(a)
        h = se * stats.t._ppf((1+confidence)/2., n-1)
        mean.append(m)
        lci.append(m-h)
        uci.append(m+h)
    return mean, lci, uci

def calc_mad(data):
    mad = []
    for idx in data.index:
        a = 1.0*np.array(data.loc[idx])
        mad.append(mean(absolute(a - mean(a)))) 
    return(mad)

#%%
path = 'Y:/Projects/ToxvalModels/'
path = 'C:/Users/Administrator/OneDrive/Profile/Desktop/ToxvalModels/'
#%%
###########################################################################
## Load normalization vector 
###########################################################################
padel_scaler = pickle.load(open(path+'finalmodels/padel_scaler.sav', 'rb'))
cdk_scaler = pickle.load(open(path+'finalmodels/cdk_scaler.sav', 'rb'))

###########################################################################
## Read Final Features from the 5-fold CV model
###########################################################################
features = pd.read_csv(path+'output/FinalModelFeatures.csv')

#%%
###########################################################################
## Predict TSCA List 
## Read fingerprint and descriptor data for chemicals to be predicted
## Final models need PubChem fingerprints, CDK and PaDEL descriptors
###########################################################################
## PubChem FPs: 881 bits
df_pubchem = pd.read_csv(path+'external/tsca/tsca_pubchem.csv', index_col='row ID')
df_pubchem = df_pubchem[~df_pubchem.index.duplicated(keep='first')]
#%%
# PaDEL
df_padel = pd.read_csv(path+'external/tsca/tsca_padel.csv', index_col='Name')
df_padel = df_padel[~df_padel.index.duplicated(keep='first')].dropna()
#%%
# Normalize PaDEl properties based on transformation scaler vector from the base models
padel_scaled = padel_scaler.transform(df_padel)
padel = pd.DataFrame(padel_scaled, index = df_padel.index) 
padel.columns = df_padel.columns
#%%
# CDK
df_cdk = pd.read_csv(path+'external/tsca/tsca_cdk.csv', index_col = 'row ID')
df_cdk = df_cdk[~df_cdk.index.duplicated(keep='first')]
# Normalize CDK properties based on transformation scaler vector from the base models
cdk_scaled = cdk_scaler.transform(df_cdk)
cdk = pd.DataFrame(cdk_scaled, index = df_cdk.index) 
cdk.columns = df_cdk.columns
#%%
## Extract the features that go into the final model
# PubChem
retain = [str(val.replace("'", "").replace(" ", "")) for val in features.ix[0,'Fingerprints'].split(',')]
retain[0] = retain[0].replace("[", "")
retain[len(retain)-1] = retain[len(retain)-1].replace("]","") ##manually check the last entry and correct it
pubchem_model = df_pubchem.loc[:,retain]
#%%
# PaDEL
retain = [str(val.replace("'", "").lstrip()) for val in features.ix[0,'PaDEL'].split(',')]
retain[0] = retain[0].replace("[", "")
retain[len(retain)-1] = retain[len(retain)-1].replace("]","") ##manually check the last entry and correct it
padel_model = padel.loc[:,retain]
#%%
# CDK
retain = [str(val.replace("'", "").lstrip()) for val in features.ix[0,'CDK'].split(',')]
retain[0] = retain[0].replace("[", "")
retain[len(retain)-1] = retain[len(retain)-1].replace("]","") ##manually check the last entry and correct it
cdk_model = cdk[retain]

#%%
###########################################################################
## Create and set the X vector for various species and study combinations
## i.e. Duplicate X vector rows and add species and study descriptor values
###########################################################################
# Determine complete list of different study and species combination
species = [0,1,2] # 0:rat, 1: mouse, 2: rabbit
studies = [0,1,2,3,4] # 0: CHR, 1: SUBCHR, 2: REP, 3: DEV, 4: SUB
study_specie = pd.DataFrame(list(itertools.product(species, studies)), columns = ['specie', 'study'])

# Set X for TSCA List
tsca_X = pd.concat([pubchem_model, padel_model, cdk_model], axis = 1).dropna()#.head(2)      
tsca_X.reset_index(inplace=True)
# Repeat each value/row to duplicate for different study and species combination
tsca_X_repeat = pd.DataFrame(np.repeat(tsca_X.values, 15, axis=0))
tsca_X_repeat.columns = tsca_X.columns

#%%
# Repeat each study and species combination for appending to TSCA X list
# i.e enable a concat for the two dataframes
study_specie_X = pd.DataFrame()
for idx in tsca_X.index:
    study_specie.index = np.repeat(idx,15)
    study_specie_X = study_specie_X.append(study_specie)
study_specie_X.reset_index(inplace=True)
study_specie_X.drop(['index'], axis=1, inplace=True)
#%%
# Final X vector for predictions
tsca_X_predict = pd.concat([tsca_X_repeat,study_specie_X], axis = 1)
idx = ['%s|%d|%d' %(tsca_X_predict.loc[idx,'index'],tsca_X_predict.loc[idx,'specie'], tsca_X_predict.loc[idx,'study']) for idx in tsca_X_predict.index]
tsca_X_predict.index = idx
tsca_X_predict.drop(['index'], axis=1, inplace=True)

#%%
############################################################################################################
#-------------------------------- Load models and predict ------------------------------------------------------#
############################################################################################################
n_models = 100
tsca_X_predictions = pd.DataFrame(index=tsca_X_predict.index)
for n in range(n_models):
    model = pickle.load(open(path+'finalmodels/model_bootstrap_%d.pkl'%n,  'rb'))
    tsca_X_predictions['Prediction Model_%d' %n] = model.predict(tsca_X_predict)

# Summarize predictions to include mean POD and 95% CI
tsca_X_predictions_copy = tsca_X_predictions.copy()
tsca_X_predictions['95% CI'] = 2*tsca_X_predictions_copy.std(axis=1)
tsca_X_predictions['Predicted POD'] = tsca_X_predictions_copy.mean(axis=1)

# Add columns to indicate the species/study combination for the predictions
tsca_X_predictions['CASRN'] = [idx.split("|")[0] for idx in tsca_X_predictions.index]
tsca_X_predictions['species'] = [idx.split("|")[1] for idx in tsca_X_predictions.index]
tsca_X_predictions['study'] = [idx.split("|")[2] for idx in tsca_X_predictions.index]

# Update the numeric value to actual value
tsca_X_predictions.loc[tsca_X_predictions['species']=='0', 'species'] = 'rat'
tsca_X_predictions.loc[tsca_X_predictions['species']=='1', 'species'] = 'mouse'
tsca_X_predictions.loc[tsca_X_predictions['species']=='2', 'species'] = 'rabbit'

tsca_X_predictions.loc[tsca_X_predictions['study']== '0', 'study'] = 'CHR'
tsca_X_predictions.loc[tsca_X_predictions['study']=='1', 'study'] = 'SUBCHR'
tsca_X_predictions.loc[tsca_X_predictions['study']=='2', 'study'] = 'REP'
tsca_X_predictions.loc[tsca_X_predictions['study']=='3', 'study'] = 'DEV'
tsca_X_predictions.loc[tsca_X_predictions['study']=='4', 'study'] = 'SAC'

#%%
# Reorder columns
columns = tsca_X_predictions.columns.tolist()
columnsorder = columns[-5:][::-1] + columns[:-5]
tsca_X_predictions = tsca_X_predictions[columnsorder]

#%%
# Summarize predictions on per chemical basis
tsca_X_predictions_byChem = tsca_X_predictions[['CASRN', 'Predicted POD', '95% CI']].groupby(['CASRN']).median()

#%%
############################################################################################################
#-------------------------------- Save the files to disk ------------------------------------------------------#
############################################################################################################
tsca_X_predictions.to_csv(path+'external/tsca/PradeepPOD-QSAR_predictions.csv')
tsca_X_predictions_byChem.to_csv(path+'external/tsca/PradeepPOD-QSAR_summarypredictions.csv')
