# -*- coding: utf-8 -*-
"""
Spyder Editor

This is a temporary script file.
"""

## Import packages
import warnings
warnings.filterwarnings('ignore')

import pandas as pd
import matplotlib.pyplot as plt
import numpy as np
import random
from scipy import stats

# Classifiers
from sklearn.linear_model import LinearRegression
from sklearn.neighbors import KNeighborsRegressor
from sklearn.ensemble import RandomForestRegressor
from sklearn.ensemble import GradientBoostingRegressor 
from sklearn import svm

# Machine learning relevant 
from sklearn.feature_selection import SelectPercentile, f_classif
from sklearn.feature_selection import RFE
from sklearn.feature_selection import VarianceThreshold 
from sklearn import preprocessing
from sklearn.model_selection import cross_validate, cross_val_predict
from sklearn.model_selection import GridSearchCV
import sklearn.metrics as sm
from sklearn.metrics import r2_score
from sklearn.model_selection import train_test_split

#%%
## User-defined functions
def selectFeatures_VarThresh(X, threshold):
    sel = VarianceThreshold(threshold=(threshold * (1 - threshold))) 
    X_sel = sel.fit_transform(X)
    # Convert it into a dataframe 
    x_tr = pd.DataFrame(X_sel, index = X.index) 
    x_tr.columns = sel.get_support(indices = True)
    return x_tr

## Remove culumns with >80% correlation
def correlation(dataset, threshold):
    col_corr = set() # Set of all the names of deleted columns
    corr_matrix = dataset.corr()
    for i in range(len(corr_matrix.columns)):
        for j in range(i):
            if corr_matrix.iloc[i, j] >= threshold:
                colname = corr_matrix.columns[i] # getting the name of column
                col_corr.add(colname)
                if colname in dataset.columns:
                    del dataset[colname] # deleting the column from the dataset
    return dataset

# Normalize descriptors: Transform variables to mean=0, variance=1
def normalizeDescriptors(X):
    x_norm = preprocessing.scale(X)
    x_norm = pd.DataFrame(x_norm, index = X.index) 
    x_norm.columns = X.columns
    return x_norm

# Univariate feature selection with F-test for feature scoring
# We use the default selection function: the 10% most significant features
def selectFeatures_perc(X, Y, percentile):
    model = SelectPercentile(f_classif, percentile)
    model = model.fit(X, Y) #convert datatype for use in the fit function
    scores = -np.log10(model.pvalues_)
    scores /= scores.max()
    X_tr = model.transform(X)
    ## Convert it into a dataframe 
    X_tr = pd.DataFrame(X_tr, index = X.index) 
    X_tr.columns = X.columns[model.get_support(indices=True)] 
    return X_tr

def selectFeatures_RFE(X, Y, n_features_to_select):
    model = LinearRegression()
    rfe = RFE(model, n_features_to_select )
    rfe = rfe.fit(X, Y) #convert datatype for use in the fit function
    X_tr = rfe.transform(X)
    ## Convert it into a dataframe 
    X_tr = pd.DataFrame(X_tr, index = X.index) 
    X_tr.columns = X.columns[rfe.get_support(indices=True)] 
    return X_tr
   
def returnparams_knn(n_fold, X, Y):
    parameters = {'weights':['uniform', 'distance'], 'n_neighbors':[3,4,5], 'algorithm': ['auto', 'ball_tree', 'kd_tree', 'brute']}
    clf = KNeighborsRegressor()
    grid_search = GridSearchCV(clf, cv = n_fold, param_grid = parameters)
    grid_search.fit(X, Y)
    knn_params = grid_search.best_params_    
    return knn_params

def returnparams_svm(n_fold, X, Y):
    parameters = {'kernel':['linear', 'rbf'], 'C':[0.1, 1, 10], 'gamma':[0.01, 0.1, 1], 'epsilon': [0.1, 1]}
    clf = svm.SVR()
    grid_search = GridSearchCV(clf, cv = 5, param_grid = parameters)
    grid_search.fit(X, Y)
    svm_params = grid_search.best_params_
    return svm_params
    
def returnparams_rf(n_fold, X, Y):
    parameters = {'n_estimators': [750, 1000, 1250, 1500], 'max_features': ['sqrt', 'auto'], 'random_state':[5]}
    clf = RandomForestRegressor()
    grid_search = GridSearchCV(clf, cv = n_fold, param_grid = parameters)
    grid_search.fit(X, Y)
    rf_params = grid_search.best_params_
    print(rf_params)
    return rf_params
    
def returnparams_gbr(n_fold, X, Y):
    parameters = {'n_estimators': [750, 1000], 'max_depth': [2,3,4], \
                  'random_state':[5], 'learning_rate': [0.01, 1], 'loss': ['ls', 'lad']}
    clf = GradientBoostingRegressor()
    grid_search = GridSearchCV(clf, cv = n_fold, param_grid = parameters)
    grid_search.fit(X, Y)
    gbr_params = grid_search.best_params_            
    return gbr_params
   
def predict_y(clf, X, Y, n_fold):
    y = cross_val_predict(clf, X = X, y = Y, cv = n_fold)
    return y

def predict_test_y(clf, X, Y, X_test):
    clf = clf.fit(X,Y)
    y = clf.predict(X_test)
    return y

        
#%%    
path = 'Z:/Projects/ToxvalModels/'
#path = 'C:/Users/ppradeep/Desktop/ToxvalModels/'
path = '/share/home/ppradeep/Projects/ToxvalModels/'
#path = '/home/ppradeep/th034_home/Projects/ToxvalModels/'
#%% Read the input data file
#data_file = pd.read_excel(path+'data/toxval_pod_summary_min_use_me_2_2017-09-21.xlsx', index_col = 'dsstox_compound_id')
data_file = pd.read_excel(path+'data/toxval_pod_summary_min_quality_id_human health_-100_2019-02-13.xlsx', index_col = 'casrn')
data_file = data_file.drop_duplicates(data_file.columns.difference(['toxval_id']))
#%%
## Read PubChem fingerprints and CDK descriptors generated in KNIME
toxprints = pd.read_csv(path+'data/toxval_toxprints.csv', index_col = 'M_NAME')
pubchem = pd.read_csv(path+'data/toxval_pubchem.csv', index_col = 'row ID')
padel = pd.read_csv(path+'data/toxval_padel.csv', index_col = 'Name')
cdk = pd.read_csv(path+'data/toxval_cdk.csv', index_col = 'row ID')
             

model = ['rat', 'mouse', 'rabbit']
rac = ['chronic', 'subchronic', 'reproductive','developmental', 'subacute']


#Y_all = data_file[(data_file['risk_assessment_class'].isin(rac)) \
#               & (data_file['toxval_units'] == 'mg/kg-day') \
#               #& (~data_file['toxval_type'].isin(['RfD', 'Rfc'])) \
#               & (data_file['toxval_type'].isin(['nel','NOEL', 'NOAEL','NOAEL ', 'NOEC','NOAEC',\
#                 'LOEL','LEL','LEC','LOEC', 'LOAEC', 'NOAEL/LOAEL'\
#                 'BMCL','BMDL','BMDL-10','BMC','BMDL-1SD','BMDL-05','BMCL-10','BMCL-5','BMCL-1SD', 'BMDL-01','BMC10',\
#                 'POD'])) \
#               #& (data_file['use_me'] > 1) \
#               #& (data_file['species'].isin(model))]['toxval_numeric']
#               & (data_file['species'].isin(model))][['toxval_numeric', 'species', 'risk_assessment_class']] #, 'risk_assessment_class', 'exposure_route'

Y_all = data_file[(data_file['risk_assessment_class'].isin(rac)) \
               & (data_file['toxval_units'] == 'mg/kg-day') \
               & (data_file['toxval_type_supercategory'] == 'Point of Departure') \
               & (~data_file['toxval_type'].isin(['ERCL10','NOTEL'])) \
               & (data_file['species_common'].isin(model))][['toxval_numeric', 'species_common', 'risk_assessment_class']] #, 'risk_assessment_class', 'exposure_route'

Y_all['toxval_numeric'] = np.log10(Y_all['toxval_numeric'])

## single POD value for all study types and species
Y1 = Y_all.groupby(Y_all.index)['toxval_numeric'].median()


## single POD value for grouped by study types and species + study type and species as desriptors in model
Y2 = Y_all
# Calculate the std for all the groups
Y2_std = Y2.groupby([Y2.index, 'risk_assessment_class', 'species_common']).std(ddof=0).reset_index()
Y2_std.rename(columns = {'toxval_numeric': 'std'}, inplace = True)
Y2_std.index = Y2_std.casrn

## calculate the median value for each group
Y2 = Y_all
Y2 = Y2.groupby([Y2.index, 'risk_assessment_class', 'species_common']).median().reset_index()
Y2.index = Y2.casrn
Y2['species'] = Y2.apply(lambda col: pd.factorize(col)[0])['species_common'] #convert species to numeric
Y2['risk_assessment_class'] = Y2.apply(lambda col: pd.factorize(col)[0])['risk_assessment_class'] #convert rac to numeric
Y2['std'] = Y2_std['std']


### Histogram of Y
#plt.figure(figsize=(8,6), dpi = 200)
#Y.hist(alpha = 0.75)
#plt.xlabel('POD', size = 30, labelpad = 20)
#plt.ylabel('Frequency', size = 30, labelpad = 20)
#plt.savefig(path+'/output/OrigDataDist_%s' %model, bbox_inches='tight')


Y = Y1
# Pubchem
X1 = pubchem.loc[Y.index,:].dropna()
# Feature Selection on X1 fingerprints using 80% variance threshold
X1 = selectFeatures_VarThresh(X1, 0.8)
# Remove culumns with >80% correlation
X1 = correlation(X1, 0.80)

# Padel
X2 = padel.loc[Y.index,:].dropna() 
## Feature selection from Padel descriptors
X2 = normalizeDescriptors(X2)
X2 = selectFeatures_RFE(X2, Y[X2.index], 5)

# cdk
X3 = cdk.loc[Y.index,:].dropna() # CDK descriptors
## Feature selection from Padel descriptors
X3 = normalizeDescriptors(X3)
X3 = selectFeatures_RFE(X3, Y[X3.index], 5)


## Model 1
# Get X by combining X1 and X2
X = pd.concat([X1,X2,X3], axis = 1).dropna() # species, r_a_c
# Update Y based on X columns that are retained
Y = Y[X.index] #Y2['toxval_numeric']
# Final Y value stats according to X data
#print stats.skew(Y)
mean1 = np.mean(Y)
sigma1 = np.std(Y)
print(np.sqrt(mean1), sigma1)


## Model 2
datatomodel = pd.concat([X1,X2,X3], axis = 1).join(Y2).dropna() # species, r_a_c
datatomodel_train, datatomodel_test = train_test_split(datatomodel, test_size = 0.20, random_state = 40)

## Model 2: Set data for bootstrapping
X_train = datatomodel_train[datatomodel_train.columns.difference(['index','toxval_numeric','casrn', 'std', 'species_common'])]
X_test = datatomodel_test[datatomodel_test.columns.difference(['index','toxval_numeric','casrn', 'std', 'species_common'])]
Y_train = datatomodel_train['toxval_numeric']
Y_test = datatomodel_test['toxval_numeric']

#%%
## Histogram of Y_train
skew = stats.skew(Y_train)
mean = np.mean(Y_train)

ax = plt.figure(figsize=(8,6), dpi = 200)
Y_train.hist(bins=10,alpha = 0.75, grid="off")
ax.text(0.15, 0.80, r'$Skew: %.2f$' %skew, fontsize = 24)
ax.text(0.15, 0.70, r'$\mu: %.2f$' %mean, fontsize = 24)
plt.xlabel(r'POD$_{tr}$', size = 30, labelpad = 20)
plt.ylabel('Frequency', size = 30, labelpad = 20)
plt.xticks(fontsize = 24)#, rotation = 90)
plt.yticks([0,500, 1000, 1500, 2000], fontsize = 24)
plt.savefig(path+'/output/BD_TrainDataDist_%s_%s_species_rac' %(model, rac), bbox_inches='tight')

#%%
# Repeat n times
n = 500
rmse = {}; r2 = {}
rmse_ext = {}; r2_ext = {}
#knn_params = []; svm_params = []; gbr_params = []
rf_params = []

mean_train = []; sigma_train = [];skew_train = []
mean_test = []; sigma_test = [];skew_test = []

Y_predicted = {}; Y_test_predicted = {};

for i in range(n):
    X = X_train.reset_index().drop(columns = 'index')
    Y = Y_train.reset_index().drop(columns = 'index')
    
    skew_Y = stats.skew(Y)
    # Duplicating data to get rid of the left skew in the Y data distribution
    n_extra = int(len(Y)/10)
    y_extra = Y[Y < Y.mean()].dropna()

    #random.seed(1983)
    random_index = random.sample(y_extra.index.tolist(), n_extra)
    y_extra = y_extra.loc[random_index]
    x_extra = X.loc[random_index, :]

    # Add duplicate data to original Y and X dataframe
    Y = Y.append(y_extra)
    X = X.append(x_extra)
 
    mean_train = np.mean(Y)
    sigma_train= np.std(Y)
    skew_train = stats.skew(Y)

    mean_test.append(np.mean(Y_test))
    sigma_test.append(np.std(Y_test))
    skew_test.append(stats.skew(Y_test))
       
#    ax = plt.figure(figsize=(8,6), dpi = 200)
#    Y.hist(bins=10, color = 'r', alpha = 0.75, grid="off", figsize=(8,6))
#    ax.text(0.15, 0.80, r'$Skew: %.2f$' % skew_train, fontsize = 24)
#    ax.text(0.15, 0.70, r'$\mu: %.2f$' %(mean_train), fontsize = 24)
#    plt.xlabel(r'POD$_{tr}$', size = 30, labelpad = 20)
#    plt.ylabel('Frequency', size = 30, labelpad = 20)
#    plt.title('')
#    plt.xticks(fontsize = 24)#, rotation = 90)
#    plt.yticks(fontsize = 24)
#    plt.savefig(path+'/output/BD_BalancedTrainDataDist_%s_%s_species_rac' %(model,rac), bbox_inches='tight')

    ## Evaluate the hyperparameters for all models based on n_fold CV
    n_fold = 5
    classifiers = [#KNeighborsRegressor(**returnparams_knn(n_fold, X, Y)),\
                   #svm.SVR(**returnparams_svm(n_fold, X, Y)),\
                   #RandomForestRegressor(**returnparams_rf(n_fold, X, Y)),\
                   RandomForestRegressor(bootstrap=True, criterion='mse', max_depth=None,
                   max_features='sqrt', max_leaf_nodes=None,
                   min_impurity_decrease=0.0, min_impurity_split=None,
                   min_samples_leaf=1, min_samples_split=2,
                   min_weight_fraction_leaf=0.0, n_estimators=1500, n_jobs=None,
                   oob_score=False, random_state=5, verbose=0, warm_start=False)
                   #GradientBoostingRegressor(**returnparams_gbr(n_fold, X, Y))     
                   ]
    for clf in classifiers:
        # 5-fold CV training
        Y_predicted[i] = predict_y(clf, X, Y, n_fold)
        rmse.setdefault(str(clf).split('(')[0], []).append(np.sqrt(sm.mean_squared_error(Y, Y_predicted[i])))
        r2.setdefault(str(clf).split('(')[0], []).append(r2_score(Y, Y_predicted[i]))
#        # Plot true vs predicted for each classifier
#        plt.figure(figsize=(8,8), dpi = 200)
#        plt.scatter(Y.values, Y_predicted[i], alpha = 0.75, color = 'r')
#        plt.plot([-6, 5],[-5, 6],'b', linestyle = '--')
#        plt.plot([-6, 5],[-7, 4],'b', label = '$\pm$1(log10) error interval', linestyle = '--')
#        plt.legend(loc='lower right', numpoints = 2, scatterpoints = 1, fontsize = 12)
#        plt.plot(Y.values, Y.values, 'k')
##        plt.xlim([Y.min(), Y.max()])
##        plt.ylim([Y.min(), Y.max()])
#    
#        plt.xlim([-2, 4])
#        plt.ylim([-2, 4])
#        plt.xlabel('Observed', size = 26, labelpad = 18)
#        plt.ylabel('Predicted', size = 26, labelpad = 18)
#        plt.savefig(path+'/output/BD_Int_TvsP_%s_%s_%s_species_rac' %(str(clf).split('(')[0], model, rac), bbox_inches='tight')

        # Fit model on entire training data and make predictions for test set
        Y_test_predicted[i] = predict_test_y(clf, X, Y, X_test)
        rmse_ext.setdefault(str(clf).split('(')[0], []).append(np.sqrt(sm.mean_squared_error(Y_test, Y_test_predicted[i])))
        r2_ext.setdefault(str(clf).split('(')[0], []).append(r2_score(Y_test, Y_test_predicted[i]))
#        # Plot true vs predicted for each classifier    
#        plt.figure(figsize=(8,8), dpi = 200)
#        plt.scatter(Y_test, Y_test_predicted[i], alpha = 0.75, color = 'g')
#        plt.plot([-6, 5],[-5, 6],'b', linestyle = '--')
#        plt.plot([-6, 5],[-7, 4],'b', label = '$\pm$1(log10) error interval', linestyle = '--')
#        plt.legend(loc='lower right', numpoints = 2, scatterpoints = 1, fontsize = 12)
#        plt.plot(Y_test, Y_test, 'k')
##        plt.xlim([Y_test.min(), Y_test.max()])
##        plt.ylim([Y_test.min(), Y_test.max()])
#        plt.xlim([-2, 4])
#        plt.ylim([-2, 4])
#        plt.xlabel('Observed', size = 26, labelpad = 18)
#        plt.ylabel('Predicted', size = 26, labelpad = 18)
#        #plt.title('Classifier: %s' %(str(clf).split('(')[0]))
#        plt.savefig(path+'/output/BD_Ext_TvsP_%s_%s_%s_species_rac' %(str(clf).split('(')[0], model, rac), bbox_inches='tight')

#%%
# Plot the balanced dataset
ax = plt.figure(figsize=(8,6), dpi = 200)
Y.iloc[:,0].hist(bins=10, color = 'r', alpha = 0.75, grid="off", figsize=(8,6))
ax.text(0.15, 0.80, r'$Skew: %.2f$' % skew_train, fontsize = 24)
ax.text(0.15, 0.70, r'$\mu: %.2f$' %(mean_train), fontsize = 24)
plt.xlabel(r'POD$_{tr}$', size = 30, labelpad = 20)
plt.ylabel('Frequency', size = 30, labelpad = 20)
plt.title('')
plt.xticks(fontsize = 24)#, rotation = 90)
plt.yticks(fontsize = 24)
plt.savefig(path+'/output/BD_BalancedTrainDataDist_%s_%s_species_rac' %(model,rac), bbox_inches='tight')
    
#%%
# Print model performance metrics
file = pd.DataFrame()
for key in rmse.keys():
    file['Classifier'] = classifiers
    file['Y_mean'] = np.mean(Y)[0]
    file['Y_std'] = np.std(Y)[0]
    file['Y_skew'] = skew_Y[0]
    file['train_mean'] = mean_train[0]
    file['train_std'] = sigma_train[0]
    file['train_skew'] = skew_train[0]
    file['RMSE_Int'] = np.average(rmse[key])
    file['RMSE_Int/sigma'] = np.average(rmse[key])/np.std(Y_train)
    file['R2_Int'] = np.average(r2[key])
    file['RMSE_Ext'] = np.average(rmse_ext[key])
    file['RMSE_Ext/sigma'] = np.average(rmse_ext[key])/np.std(Y_test)
    file['R2_Ext'] = np.average(r2_ext[key])
file.to_csv(path+'/output/BD_ModelInfo_%s_%s_%s_species_rac.csv' %(str(clf).split('(')[0], rac, model))

#%%
## For the poster
# Plot true vs predicted for each classifier (Internal)
ax = plt.figure(figsize=(10,8), dpi = 200)
plt.scatter(Y.values, Y_predicted[i], alpha = 0.75, color = 'r')
plt.plot([-6, 5],[-5, 6],'b', linestyle = '--')
plt.plot([-6, 5],[-7, 4],'b', label = '$\pm$1(log10) error interval', linestyle = '--')
plt.legend(loc='lower right', numpoints = 2, scatterpoints = 1, fontsize = 14)
plt.plot(Y.values, Y.values, 'k')

ax.text(0.15, 0.80, r'$Avg. RMSE: %.2f$' % np.average(rmse[key]), fontsize = 22)
ax.text(0.15, 0.75, r'$Avg. R^{2}: %.2f$' % np.average(r2[key]), fontsize = 22)

plt.xlim([-2, 4])
plt.ylim([-2, 4])
plt.xticks(fontsize = 28, rotation = 90)
plt.yticks(fontsize = 28)
plt.xlabel('Observed', size = 28, labelpad = 18)
plt.ylabel('Predicted', size = 28, labelpad = 18)
plt.savefig(path+'/output/BD_Int_TvsP_%s_%s_%s_species_rac1' %(str(clf).split('(')[0], model, rac), bbox_inches='tight')
        
# Plot true vs predicted for each classifier(External) 
ax = plt.figure(figsize=(10,8), dpi = 200)
plt.scatter(Y_test, Y_test_predicted[i], alpha = 0.75, color = 'g')
plt.plot([-6, 5],[-5, 6],'b', linestyle = '--')
plt.plot([-6, 5],[-7, 4],'b', label = '$\pm$1(log10) error interval', linestyle = '--')
plt.legend(loc='lower right', numpoints = 2, scatterpoints = 1, fontsize = 14)
plt.plot(Y_test, Y_test, 'k')

ax.text(0.15, 0.80, r'$Avg. RMSE: %.2f$' % np.average(rmse_ext[key]), fontsize = 22)
ax.text(0.15, 0.75, r'$Avg. R^{2}: %.2f$' % np.average(r2_ext[key]), fontsize = 22)

plt.xlim([-2, 4])
plt.ylim([-2, 4])
plt.xticks(fontsize = 28, rotation = 90)
plt.yticks(fontsize = 28)
plt.xlabel('Observed', size = 28, labelpad = 18)
plt.ylabel('Predicted', size = 28, labelpad = 18)
plt.savefig(path+'/output/BD_Ext_TvsP_%s_%s_%s_species_rac1' %(str(clf).split('(')[0], model, rac), bbox_inches='tight')     